/**
 * This class contains generic functions shared across all tabs.
 */
function GenericTab() {
    // PRIVATE MEMBERS \\       var private = "test";
    var that = this;
    var activeClass = "tabactive";

    // PRIVATE FUNCTIONS \\     function privateFunction(){}

    /**
     * Removes the active tab class from all elements that currently possess it.
     */
    function removeActiveClass() {
        $("." + activeClass).removeClass(activeClass);
    }

    /**
     * Adds the active tab class to the element with the passed id.
     * @param {string} id The id of the new active tab button.
     */
    function addActiveClass(id) {
        $("#" + id).addClass(activeClass);
    }

    // PUBLIC MEMBERS \\        this.public = "test";
    this.saveSlotClass = "saveslot";
    this.smallSaveSlotClass = "smallsaveslot";
    this.saveSlotSelectedClass = "saveslotselected";

    // PUBLIC FUNCTIONS \\      this.publicFunction = function(){};

    /**
     * Transitions the UI from one tab button to another.
     * @param {string} id The id of the new active tab button.
     */
    this.tabButtonTransition = function (id) {
        //global.ui.cancelThemeChangeTransition(); //Instantly completes any theme-changes in progress.
        removeActiveClass();
        addActiveClass(id);
    };

    /**
     * Returns true if the tab with the passed id is active. False otherwise.
     * @param {string} id The id of the tab to check.
     * @returns {boolean} True if the tab with the passed id is active. False otherwise.
     */
    this.isActive = function (id) {
        return $("#" + id).hasClass(activeClass);
    };

    /*
     * Builds and returns the top right maximum save indicator.
     * @param {array} saveMeta The list of save metadata. 
     * @returns {string} The maximum save indicator.
     */
    this.buildMaxSaveIndicator = function (saveMeta) {
        var amount = saveMeta.length;
        var topRight = `<div class="savelimit">${amount} / ${global.saveManager.maxAmountOfSaves}</div>`;
        return topRight;
    };

    /**
     * Updates the maximum save indicator to new values.
     */
    this.updateMaxSaveIndicator = function () {
        var amount = global.saveManager.getAmountOfSaves();
        $(".savelimit").html(`${amount} / ${global.saveManager.maxAmountOfSaves}`);
    };

    /**
     * Converts a timestamp to a string-representation of how long ago it was.
     * @param {Date} time The timestamp.
     * @returns {string} A string-representation of how long ago the timestamp was.
     */
    this.timeToTimePassed = function (time) {
        var seconds = Math.floor((new Date() - time) / 1000);
        var interval = Math.floor(seconds / 31536000);
        if (interval > 1) {
            return interval + " years ago";
        }
        interval = Math.floor(seconds / 2592000);
        if (interval > 1) {
            return interval + " months ago";
        }
        interval = Math.floor(seconds / 86400);
        if (interval > 1) {
            return interval + " days ago";
        }
        interval = Math.floor(seconds / 3600);
        if (interval > 1) {
            return interval + " hours ago";
        }
        interval = Math.floor(seconds / 60);
        if (interval > 1) {
            return interval + " minutes ago";
        }
        interval = seconds;
        if (interval > 10) {
            return interval + " seconds ago";
        }
        return "Just now";
    };

    /**
     * Removes the saveslotselected class from all elements.
     */
    this.removeSaveSlotSelected = function () {
        $("." + that.saveSlotSelectedClass).removeClass(that.saveSlotSelectedClass);
    };

    /**
     * Enables the element with the passed id.
     * @param {string} elementId The id of the element.
     */
    this.enable = function (elementId) {
        $("#" + elementId).prop("disabled", false);
    };

    /**
     * Disables the element with the passed id.
     * @param {string} elementId The id of the element.
     */
    this.disable = function (elementId) {
        $("#" + elementId).prop("disabled", true);
    };

    /**
     * Builds a generic header to be used for every tab apart from scene.
     * @param {string} title The title of the tab.
     * @returns {string} A generic tab header.
     */
    this.buildTabHeader = function (title) {
        var start = global.ui.buildGenericHeader();
        return start + "<div class='title'>" + title + "</div>";
    };

    /**
     * Builds a generic footer to be used for every tab apart from scene.
     * @returns {string} A generic tab footer.
     */
    this.buildTabFooter = function () {
        return global.ui.buildGenericFooter();
    };

    /**
     * Resets the scroll position.
     */
    this.resetScroll = function () {
        $("#main").scrollTop(0);
    };
}
