/**
 * The path to home of the Forest scene.
 * @param {ForestController} controller The forest controller.
 * @param {ForestBandit} toBandit A part the kobold may go to next.
 * @param {ForestHomeHide} toHide A part the kobold may go to next.
 * @param {ForestHomeIgnore} toIgnore A part the kobold may go to next.
 */
function ForestHomePath(controller, toBandit, toHide, toIgnore) {
    // PRIVATE MEMBERS \\       var private = "test";
    var that = this;
    var ctrl = controller;
    var bandit = toBandit;
    var hide = toHide;
    var ignore = toIgnore;

    var investigateId = "homeInvestigateButton";
    var hideId = "homeHideButton";
    var ignoreId = "homeIgnoreButton";

    var content = ` <div class="text">
                        You're not too far away from your humble homestead.
                        You decide to go there, since there's bound to be some kind of debt collector already waiting on your porch.
                        And if there isn't, at least you can rest your sore legs on your shabby bed for a bit.
                        In fact, your entire body feels kind of stiff, burdened by overexertion and the rollercoaster of emotions you've been through today.
                        It'd be nice to be able to wind down in a somewhat comfortable, familiar place, if just for long enough to collect yourself.
                    </div>
                    <div class="text">
                        You head for the nearby <span class="bold">eastern Varanar road</span>, which differs from the southern one, 
                        which is where you robbed the money transport.
                        Before crossing, you cautiously look from side to side, scouting for anyone that may cause trouble.
                        Luckily for you, this is one of the least travelled roads to the capital, and hence, it is relatively unplagued by highwaymen.
                        Still, you leave little up to chance.
                        You bolt across the path, moving from one side to the other in the flash of an eye.
                        Once again you check your surroundings.
                        Nobody. 
                        Phew.
                    </div>
                    <div class="text">
                        Walking alongside the road to maintain some modicum of stealth, you quickly come upon the small, well-hidden path that leads to your shack.
                        All of a sudden, a loud scream echoes across the forest.
                        It sounds like a woman in distress.
                    `;

    var contentEnd = `  fully expecting an ambush. 
                        It is only after you've fully prepared yourself for battle, both physically and mentally, 
                        that you come to the conclusion it was more of a distant wail instead of a nearby rallying cry.
                        You realize you're more than a little overstressed, teetering <span class="italic">on the edge</span>.
                        You slip out of your fighting pose.
                        That shout scared the <span class="bold">hell</span> out of you.
                    </div>`;

    // PRIVATE FUNCTIONS \\     function privateFunction(){}

    /**
     * Builds and returns the weapon dependent part of this section.
     * @returns {string} The weapon dependent part of this section.
     */
    function buildWeaponDependentPart() {
        var weapon = ctrl.home.pathWeapon();
        if (weapon) {
            return ` You ready your weapon, `;
        } else {
            return ` You drop into a combat stance, `;
        }
    }

    /**
     * Builds and returns the main content of this section.
     * @returns {string} The main content of this section.
     */
    function buildContent() {
        return content + buildWeaponDependentPart() + contentEnd;
    }

    /**
     * Builds and returns the investigate choice button.
     */
    function buildInvestigateChoice() {
        var disabled = ctrl.home.hasMadePathChoice() ? "disabled" : "";
        var isSelected = ctrl.home.getPathChoice() == ForestHomePathChoiceEnum.INVESTIGATE;
        var selected = isSelected ? "selected" : "";
        var button = `<button class="button inlinechoice ${selected}" id="${investigateId}" ${disabled}>Investigate</button>`;

        var tooltip = `Investigate the shriek. Someone may be in trouble!`;
        var popupContainer = global.ui.makeToolTip(button, tooltip);
        // Re-wrap with proper positioning.
        button = `<div class="inlinechoice">`;
        button += popupContainer;
        button += `</div>`;

        return button;
    }

    /**
     * Builds and returns the hide choice button.
     */
    function buildHideChoice() {
        var disabled = ctrl.home.hasMadePathChoice() ? "disabled" : "";
        var isSelected = ctrl.home.getPathChoice() == ForestHomePathChoiceEnum.HIDE;
        var selected = isSelected ? "selected" : "";
        var button = `<button class="button inlinechoice ${selected}" id="${hideId}" ${disabled}>Hide</button>`;

        var tooltip = `That wail was terrifying! Hide from whatever may have uttered it.`;
        var popupContainer = global.ui.makeToolTip(button, tooltip);
        // Re-wrap with proper positioning.
        button = `<div class="inlinechoice">`;
        button += popupContainer;
        button += `</div>`;

        return button;
    }

    /**
     * Builds and returns the ignore choice button.
     */
    function buildIgnoreChoice() {
        var disabled = ctrl.home.hasMadePathChoice() ? "disabled" : "";
        var isSelected = ctrl.home.getPathChoice() == ForestHomePathChoiceEnum.CONTINUE;
        var selected = isSelected ? "selected" : "";
        var button = `<button class="button inlinechoice ${selected}" id="${ignoreId}" ${disabled}>Continue</button>`;

        var tooltip = `Ignore the scream and continue on your way home.`;
        var popupContainer = global.ui.makeToolTip(button, tooltip);
        // Re-wrap with proper positioning.
        button = `<div class="inlinechoice">`;
        button += popupContainer;
        button += `</div>`;

        return button;
    }

    /**
     * Builds and returns the choice of this section.
     * @returns {string} The choice of this section.
     */
    function buildChoice() {
        var choice = `<div class="choicecontainer"><div class="inlinechoiceheader">You:</div>`;
        choice += buildInvestigateChoice();
        choice += `<hr class="inlinechoiceseparator">`;
        choice += buildHideChoice();
        choice += `<hr class="inlinechoiceseparator">`;
        choice += buildIgnoreChoice();
        choice += `</div>`;
        return choice;
    }

    /**
     * Disables all three buttons.
     */
    function disableButtons() {
        var buttons = [$("#" + investigateId), $("#" + hideId), $("#" + ignoreId)];
        buttons.forEach(function (button) {
            button.unbind();
            button.attr("disabled", true);
        });
    }

    /**
     * Loads and displays the next part.
     * @param {ForestHomePathChoiceEnum} next Which part to load.
     */
    function loadNextPart(next) {
        var part;
        if (next === ForestHomePathChoiceEnum.INVESTIGATE) {
            part = bandit;
        }
        if (next === ForestHomePathChoiceEnum.HIDE) {
            part = hide;
        }
        if (next === ForestHomePathChoiceEnum.CONTINUE) {
            part = ignore;
        }
        var built = part.build(true);
        global.ui.appendBeforeFinisher(built, global.ui.genericHeaderId);
        $("#" + part.id).fadeIn(400);
        part.registerListeners();
    }

    /**
     * Adds a listener to the investigate button.
     */
    function registerInvestigateListener() {
        $("#" + investigateId).click(function () {
            $(this).addClass("selected");
            disableButtons();
            ctrl.home.makePathChoice(ForestHomePathChoiceEnum.INVESTIGATE);
            ctrl.bandit.setSource(ForestBanditSourceEnum.HOME);
            loadNextPart(ForestHomePathChoiceEnum.INVESTIGATE);
        });
    }

    /**
     * Adds a listener to the hide button.
     */
    function registerHideListener() {
        $("#" + hideId).click(function () {
            $(this).addClass("selected");
            disableButtons();
            ctrl.home.makePathChoice(ForestHomePathChoiceEnum.HIDE);
            loadNextPart(ForestHomePathChoiceEnum.HIDE);
        });
    }

    /**
     * Adds a listener to the ignore button.
     */
    function registerIgnoreListener() {
        $("#" + ignoreId).click(function () {
            $(this).addClass("selected");
            disableButtons();
            ctrl.home.makePathChoice(ForestHomePathChoiceEnum.CONTINUE);
            loadNextPart(ForestHomePathChoiceEnum.CONTINUE);
        });
    }

    // PUBLIC MEMBERS \\        this.public = "test";
    this.id = "homePath";

    // PUBLIC FUNCTIONS \\      this.publicFunction = function(){};

    /**
     * Builds and returns this part.
     * @param {boolean} invisible If passed and true, will give the entire section a display:none style.
     */
    this.build = function (invisible) {
        var change = ctrl.home.pathStatusChange();
        var part = `<div id="${that.id}" ${invisible ? 'style="display:none;"' : ""}>`;
        part += buildContent();
        part += change;
        part += buildChoice();
        part += `</div>`;
        if (ctrl.home.getPathChoice() == ForestHomePathChoiceEnum.INVESTIGATE) {
            part += bandit.build();
        }
        //part += hide.build();   // Build by ForestHome
        //part += ignore.build(); // Build by ForestHome
        return part;
    };

    /**
     * Registers all the required button listeners for this part.
     */
    this.registerListeners = function () {
        if (ctrl.home.getPathChoice() == ForestHomePathChoiceEnum.INVESTIGATE) {
            bandit.registerListeners();
        }

        if (ctrl.home.hasMadePathChoice()) {
            return;
        }
        registerInvestigateListener();
        registerHideListener();
        registerIgnoreListener();
    };
}

/**
 * Path choices to be made.
 */
var ForestHomePathChoiceEnum = {
    INVESTIGATE: 1,
    HIDE: 2,
    CONTINUE: 3
};
